/*
 * {main.c}
 *
 * {main application}
 *
 * Copyright (C) 2013 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
//******************************************************************************
//  RF430 NFC Booster Pack (Host/Master)
//
//  RF430 User Address Map
//  -----------------------------------------
//  Address		| Size	| Description		|
//  -----------------------------------------
//  0xFFFE		| 2B	| Control Register	|
//  0xFFFC		| 2B	| Status Register	|
//  0xFFFA		| 2B	| Interrupt Enable	|
//  0xFFF8		| 2B	| Interrupt Flags	|
//  0xFFF6		| 2B	| CRC Result		|
//  0xFFF4		| 2B	| CRC Length		|
//  0xFFF2		| 2B	| CRC Start Address	|
//  0xFFF0		| 2B	| Comm WD Ctrl Reg	|
//  -----------------------------------------
//  0x0000 - 	| 2kB	| NDEF App Memory	|
//    0x07FF	|		|					|
//
//  Description: This demo demonstrates RF430 read/write accesses via the I2C bus.
//	This is the code for the tester/host processor, which is the MASTER. This master
//  can send read and write commands over I2C to access the data structure in the
//	RF430 device. The following formats are used:
//
//	Write Access:
//	The master transmits a 2 byte address, then transmits N 16 bit values MSB first
//	to be stored by RF430 in locations starting at this address and incrementing.
//
//	Read Access:
//	The master transmits a 2 byte address, then receives back from RF430
//  the two bytes RF430 has stored at this location, MSB first.
//
//	Continuous Read Access:
//	The master receives the N bytes stored in RF430 starting at the next address
//	(next address = the last accessed address + 2) and incrementing, MSB first.
//
//
//                                /|\  /|\	  (Host/Tester)
//                   RF430        10k  10k     MSP430G2553
//                  (Slave)        |    |        Master
//             _________________   |    |   _________________
//            |              SDA|<-|----+->|P1.7/UCB0SDA  XIN|-+
//            |                 |  | I2C   |                 |
//            |              SCL|<-+------>|P1.6/UCB0SCL XOUT|-
//            |                 |          |                 |
//      GND<--|E(2-0)       /RST|<---------|P1.3*            | (* NOTE: Will be moved on next hardware revision.)
//            |             INTO|--------->|P2.7*            |
//            |                 |          |             P1.0|---->LED1
//            |                 |          |                 |
//            |_________________|          |_________________|
//
// Built with CCSv5.2
//******************************************************************************

#include "msp430.h"
#include "RF430_example.h"

unsigned char NDEF_Application_Data[] = RF430_DEFAULT_DATA;

unsigned char test_data[] = {0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF};

unsigned char CRC_Data[] = {1,2,3,4,5,6,7,8,9};

unsigned char Cmd = 0;	//Command byte for SPI mode
unsigned char read_complete = 0;
unsigned char rx_byte_count = 0;
unsigned char tx_byte_count = 0;
unsigned int Results[11] = {0,0,0,0,0,0,0,0,0,0,0};

/****************************************************************************/
/* Code-binary that opens on ETW and re-trims LF oscillator to below 280kHz */
/****************************************************************************/
unsigned char func_retrim_osc[] = {
        0xB2, 0x40, 0x11, 0x96, 0x10, 0x01,
        0xB2, 0x40, 0x60, 0x03, 0x18, 0x01,
        0x30, 0x41
};

unsigned char into_fired = 0;

unsigned char read_data[200];

void main (void)
{
	volatile unsigned int test = 0;
	unsigned int flags = 0;
	unsigned int temp = 0;
	//unsigned char read_data[200];

    WDTCTL = WDTPW + WDTHOLD;				// Turn off Watch Dog

    if (CALBC1_8MHZ==0xFF)					// If calibration constant erased
    {
      while(1);                             // Do not load, trap CPU!!
    }
    BCSCTL1 = CALBC1_8MHZ;                  // Set range
    DCOCTL = CALDCO_8MHZ;					// SMCLK = MCLK = DCO = 8 MHz	ACLK = LF oscillator

    // Configure pins for I2C
	PORT_I2C_SEL0 |= (SCL + SDA);			// Selecting I2C pin function
    PORT_I2C_SEL1 |= (SCL + SDA);

    //configure USCI for I2C
	UCB0CTL1 |= UCSWRST;	            	// Software reset enabled
	UCB0CTL0 |= UCMODE_3  + UCMST + UCSYNC; // I2C mode, Master mode, sync
	UCB0CTL1 |= UCSSEL_3 + UCTR;            // SMCLK = 8MHz, transmitter
	UCB0BR0 = 80; 							// Baudrate = SMLK/80 = 100kHz
	UCB0I2CSA  = 0x0028;		    		// slave address - determined by pins E0, E1, and E2 on the RF430CL330H
	UCB0CTL1  &= ~UCSWRST;					// Software reset released


	// RST RF430 (in case board remained powered but the MSP430 reset for some reason - MSP430 RST button pushed for example)
	PORT_RST_SEL0 &= ~RST;					// Setting as GPIO pin functionality
	PORT_RST_SEL1 &= ~RST;
	PORT_RST_OUT &= ~RST;
	PORT_RST_DIR |= RST;					// RF430CL330H device in Reset
	__delay_cycles(1000);
	PORT_RST_OUT |= RST; 					// Release the RF430CL330H from Reset

	//configure LED1
	PORT_LED_SEL0 &= ~LED1; 				// Confirming GPIO pin functionality
	PORT_LED_SEL1 &= ~LED1;
	PORT_LED_DIR |= LED1; 					// Set as Output
	PORT_LED_OUT &= ~LED1; 					// Start LED off

	//configure pin for INTO interrupt
	PORT_INTO_SEL0 &= ~INTO; 				// Confirming GPIO pin functionality
	PORT_INTO_SEL1 &= ~INTO;
	PORT_INTO_DIR &= ~INTO; 				// Set as Input
	PORT_INTO_OUT |= INTO; 					// Set output register high
	PORT_INTO_REN |= INTO; 					// Internal pull-up resistor
	PORT_INTO_IFG &= ~INTO; 				// Clear interrupt flag
	PORT_INTO_IES |= INTO; 					// Set interrupt trigger as high-to-low transition, since INTO will be setup active low below

	__delay_cycles(4000000); 				// Leave time for the RF430CL33H to get itself initialized; should be 20ms or greater

	while(!(Read_Register(STATUS_REG) & READY)); // Wait until READY bit has been set

    /****************************************************************************/
    /* This code is used to change data rate capability                         */
    /****************************************************************************/

//	Write_Register(TEST_MODE_REG, TEST_MODE_KEY);   //unlock test mode
//	Write_Register(CONTROL_REG, TEST430_ENABLE);    //enable test mode, now have to use actual addresses
//	Write_Register(0x2a7C, 0xF7);                   //the bit rate capability setting, 0xF7=848kbits
//	Write_Register(0x2814, 0);                      //exit test mode (CONTROL_REG is at real address 0x2814)
//	Write_Register(TEST_MODE_REG, 0);               //exiting this mode, RF is disabled as Control register is set to 0


	    /****************************************************************************/
	    /* Configure RF430CL330H for Typical Usage Scenario                         */
	    /****************************************************************************/

    //write NDEF memory with Capability Container + NDEF message
    Write_Continuous(0, NDEF_Application_Data, 48);

    //Enable interrupts for End of Read and End of Write
    Write_Register(INT_ENABLE_REG, EOW_INT_ENABLE + EOR_INT_ENABLE);

    //Configure INTO pin for active low and enable RF
    Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE + RF_ENABLE);

	PORT_INTO_IFG &= ~INTO; 		//clear any pending flags
	PORT_INTO_IE |= INTO;			//enable interrupt

    while (1)
    {

    	__bis_SR_register(LPM3_bits + GIE); //go to low power mode and enable interrupts. We are waiting for an NFC read or write of/to the RF430
    	__no_operation();

    	//device has woken up, check status
    	if(into_fired)
    	{
    		//before we read/write to RF430CL330H we should disable RF
    		Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE); //clear control reg to disable RF
    		__delay_cycles(750000);
    		flags = Read_Register(INT_FLAG_REG); //read the flag register to check if a read or write occurred
    		Write_Register(INT_FLAG_REG, EOW_INT_FLAG + EOR_INT_FLAG); //ACK the flags to clear

       		Write_Register(INT_ENABLE_REG, 0);


			PORT_LED_OUT &= ~LED1;			//clear LEDs

    		if(flags & EOW_INT_FLAG) //check if the tag was written
    		{
    			//tag was updated, so we should read out the new data
    			//read out the data
    			//Read_Continuous(0, read_data, 200);
   			    __no_operation(); //breakpoint here to examine the data

    			//show that tag was written by blinking LED 3 times
   			    for(temp = 0; temp < 3; temp++){
   			    	PORT_LED_OUT |= LED1;
   			    	__delay_cycles(3000000);
   			    	PORT_LED_OUT &= ~LED1;//clear LED
   			    	__delay_cycles(3000000);
   			    }
    		}
    		else if(flags & EOR_INT_FLAG) //check if the tag was read
    		{
    			__no_operation();

    			//show that tag was read with LED by holding it on for 1 second
     			PORT_LED_OUT |= LED1;
    			__delay_cycles(8000000);
    			PORT_LED_OUT &= ~LED1;//clear LED
    		}

    		flags = 0;
    		into_fired = 0; //we have serviced INTO

    		//Enable interrupts for End of Read and End of Write
    		 Write_Register(INT_ENABLE_REG, EOW_INT_ENABLE + EOR_INT_ENABLE);
    		 test = Read_Register(INT_ENABLE_REG);
			//Configure INTO pin for active low and re-enable RF
			Write_Register(CONTROL_REG, INT_ENABLE + INTO_DRIVE + RF_ENABLE);

    		//re-enable INTO
    		PORT_INTO_IFG &= ~INTO;
    		PORT_INTO_IE |= INTO;

			__no_operation();
    	}
    }
}

#pragma vector=PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
	//INTO interrupt fired
	if(PORT_INTO_IFG & INTO)
	{

		into_fired = 1;

		PORT_INTO_IE &= ~INTO; //disable INTO
		PORT_INTO_IFG &= ~INTO; //clear interrupt flag

		__bic_SR_register_on_exit(LPM3_bits); //wake up to handle INTO
	}
}

#pragma vector = PORT1_VECTOR,ADC10_VECTOR,WDT_VECTOR,NMI_VECTOR,TIMER0_A1_VECTOR, \
		   TIMER0_A0_VECTOR, COMPARATORA_VECTOR, TIMER1_A1_VECTOR, TIMER1_A0_VECTOR, \
		   USCIAB0TX_VECTOR, USCIAB0RX_VECTOR
 __interrupt void ISR_trap(void)
 {
   while(1);  // IT'S A TRAP!!!
 }

